@@ -35,6 +35,7 @@
 #include <cstddef>
 #include <cstdlib>
 #include <cstring>
+#include <cfloat>
 #include <vector>
 #include <algorithm>
 
@@ -52,7 +53,45 @@
 #include "ogr_spatialref.h"
 #include "ogrsf_frmts.h"
 
-CPL_CVSID("$Id: gdalrasterize.cpp 9ff327806cd64df6d73a6c91f92d12ca0c5e07df 2018-04-07 20:25:06 +0200 Even Rouault $")
+CPL_CVSID("$Id: gdalrasterize.cpp e40d0a17ddb8ca683d53dd68777a74094f8f1bde 2019-03-12 13:31:28 +0100 Even Rouault $")
+
+
+/************************************************************************/
+/*                        gvBurnScanlineBasic()                         */
+/************************************************************************/
+template<typename T>
+static inline
+void gvBurnScanlineBasic( GDALRasterizeInfo *psInfo,
+                          int nY, int nXStart, int nXEnd,
+                          double dfVariant )
+
+{
+    for( int iBand = 0; iBand < psInfo->nBands; iBand++ )
+    {
+        const double burnValue = ( psInfo->padfBurnValue[iBand] +
+                ( (psInfo->eBurnValueSource == GBV_UserBurnValue)?
+                            0 : dfVariant ) );
+
+        unsigned char *pabyInsert = psInfo->pabyChunkBuf
+                                    + iBand * psInfo->nBandSpace
+                                    + nY * psInfo->nLineSpace + nXStart * psInfo->nPixelSpace;
+        int nPixels = nXEnd - nXStart + 1;
+        if( psInfo->eMergeAlg == GRMA_Add ) {
+            while( nPixels-- > 0 ) 
+            {
+                *reinterpret_cast<T*>(pabyInsert) += static_cast<T>(burnValue);
+                pabyInsert += psInfo->nPixelSpace;
+            }
+        } else {
+            while( nPixels-- > 0 ) 
+            {
+                *reinterpret_cast<T*>(pabyInsert) = static_cast<T>(burnValue);
+                pabyInsert += psInfo->nPixelSpace;
+            }
+        }
+    }
+}
+
 
 /************************************************************************/
 /*                           gvBurnScanline()                           */
@@ -62,11 +101,11 @@ void gvBurnScanline( void *pCBData, int nY, int nXStart, int nXEnd,
                      double dfVariant )
 
 {
+    GDALRasterizeInfo *psInfo = static_cast<GDALRasterizeInfo *>(pCBData);
+
     if( nXStart > nXEnd )
         return;
 
-    GDALRasterizeInfo *psInfo = static_cast<GDALRasterizeInfo *>(pCBData);
-
     CPLAssert( nY >= 0 && nY < psInfo->nYSize );
     CPLAssert( nXStart <= nXEnd );
     CPLAssert( nXStart < psInfo->nXSize );
@@ -78,58 +117,66 @@ void gvBurnScanline( void *pCBData, int nY, int nXStart, int nXEnd,
     if( nXEnd >= psInfo->nXSize )
         nXEnd = psInfo->nXSize - 1;
 
-    if( psInfo->eType == GDT_Byte )
+    switch (psInfo->eType)
     {
-        for( int iBand = 0; iBand < psInfo->nBands; iBand++ )
-        {
-            unsigned char nBurnValue = static_cast<unsigned char>
-                ( psInfo->padfBurnValue[iBand] +
-                  ( (psInfo->eBurnValueSource == GBV_UserBurnValue)?
-                             0 : dfVariant ) );
-
-            unsigned char *pabyInsert =
-                psInfo->pabyChunkBuf
-                + iBand * psInfo->nXSize * psInfo->nYSize
-                + nY * psInfo->nXSize + nXStart;
-
-            if( psInfo->eMergeAlg == GRMA_Add ) {
-                int nPixels = nXEnd - nXStart + 1;
-                while( nPixels-- > 0 )
-                    *(pabyInsert++) += nBurnValue;
-            } else {
-                memset( pabyInsert, nBurnValue, nXEnd - nXStart + 1 );
-            }
-        }
-    }
-    else if( psInfo->eType == GDT_Float64 )
-    {
-        for( int iBand = 0; iBand < psInfo->nBands; iBand++ )
-        {
-            int nPixels = nXEnd - nXStart + 1;
-            const double dfBurnValue =
-                ( psInfo->padfBurnValue[iBand] +
-                  ( (psInfo->eBurnValueSource == GBV_UserBurnValue)?
-                             0 : dfVariant ) );
-
-            double *padfInsert =
-                (reinterpret_cast<double *>(psInfo->pabyChunkBuf))
-                + iBand * psInfo->nXSize * psInfo->nYSize
-                + nY * psInfo->nXSize + nXStart;
-
-            if( psInfo->eMergeAlg == GRMA_Add ) {
-                while( nPixels-- > 0 )
-                    *(padfInsert++) += dfBurnValue;
-            } else {
-                while( nPixels-- > 0 )
-                    *(padfInsert++) = dfBurnValue;
-            }
-        }
-    }
-    else {
-        CPLAssert(false);
+        case GDT_Byte:
+            gvBurnScanlineBasic<GByte>( psInfo, nY, nXStart, nXEnd, dfVariant );
+            break;
+        case GDT_Int16:
+            gvBurnScanlineBasic<GInt16>( psInfo, nY, nXStart, nXEnd, dfVariant );
+            break;
+        case GDT_UInt16:
+            gvBurnScanlineBasic<GUInt16>( psInfo, nY, nXStart, nXEnd, dfVariant );
+            break;
+        case GDT_Int32:
+            gvBurnScanlineBasic<GInt32>( psInfo, nY, nXStart, nXEnd, dfVariant );
+            break;
+        case GDT_UInt32:
+            gvBurnScanlineBasic<GUInt32>( psInfo, nY, nXStart, nXEnd, dfVariant );
+            break;
+        case GDT_Float32:
+            gvBurnScanlineBasic<float>( psInfo, nY, nXStart, nXEnd, dfVariant );
+            break;
+        case GDT_Float64:
+            gvBurnScanlineBasic<double>( psInfo, nY, nXStart, nXEnd, dfVariant );
+            break;
+        default:
+            CPLAssert(false);
+            break;
     }
 }
 
+/************************************************************************/
+/*                        gvBurnPointBasic()                            */
+/************************************************************************/
+template<typename T>
+static inline
+void gvBurnPointBasic( GDALRasterizeInfo *psInfo,
+                       int nY, int nX, double dfVariant )
+
+{
+    constexpr double dfMinVariant = std::numeric_limits<T>::lowest();
+    constexpr double dfMaxVariant = std::numeric_limits<T>::max();
+
+    for( int iBand = 0; iBand < psInfo->nBands; iBand++ )
+    {
+        double burnValue = ( psInfo->padfBurnValue[iBand] +
+                ( (psInfo->eBurnValueSource == GBV_UserBurnValue)?
+                            0 : dfVariant ) );
+        unsigned char *pbyInsert = psInfo->pabyChunkBuf
+                                 + iBand * psInfo->nBandSpace
+                                 + nY * psInfo->nLineSpace + nX * psInfo->nPixelSpace;
+
+        T* pbyPixel = reinterpret_cast<T*>(pbyInsert);
+        burnValue += ( psInfo->eMergeAlg != GRMA_Add ) ? 0 : *pbyPixel;
+        *pbyPixel = static_cast<T>(
+                    ( dfMinVariant > burnValue ) ? dfMinVariant :
+                    ( dfMaxVariant < burnValue ) ? dfMaxVariant :
+                    burnValue );
+    }
+}
+
+
 /************************************************************************/
 /*                            gvBurnPoint()                             */
 /************************************************************************/
@@ -142,52 +189,31 @@ void gvBurnPoint( void *pCBData, int nY, int nX, double dfVariant )
     CPLAssert( nY >= 0 && nY < psInfo->nYSize );
     CPLAssert( nX >= 0 && nX < psInfo->nXSize );
 
-    if( psInfo->eType == GDT_Byte )
+    switch( psInfo->eType )
     {
-        for( int iBand = 0; iBand < psInfo->nBands; iBand++ )
-        {
-            unsigned char *pbyInsert = psInfo->pabyChunkBuf
-                                      + iBand * psInfo->nXSize * psInfo->nYSize
-                                      + nY * psInfo->nXSize + nX;
-            double dfVal;
-            if( psInfo->eMergeAlg == GRMA_Add ) {
-                dfVal = *pbyInsert + ( psInfo->padfBurnValue[iBand] +
-                          ( (psInfo->eBurnValueSource == GBV_UserBurnValue)?
-                             0 : dfVariant ) );
-            } else {
-                dfVal = psInfo->padfBurnValue[iBand] +
-                          ( (psInfo->eBurnValueSource == GBV_UserBurnValue)?
-                             0 : dfVariant );
-            }
-            if( dfVal > 255.0 )
-                *pbyInsert = 255;
-            else if( dfVal < 0.0 )
-                *pbyInsert = 0;
-            else
-                *pbyInsert = static_cast<unsigned char>( dfVal );
-        }
-    }
-    else if( psInfo->eType == GDT_Float64 )
-    {
-        for( int iBand = 0; iBand < psInfo->nBands; iBand++ )
-        {
-            double *pdfInsert = reinterpret_cast<double *>(psInfo->pabyChunkBuf)
-                                + iBand * psInfo->nXSize * psInfo->nYSize
-                                + nY * psInfo->nXSize + nX;
-
-            if( psInfo->eMergeAlg == GRMA_Add ) {
-                *pdfInsert += ( psInfo->padfBurnValue[iBand] +
-                         ( (psInfo->eBurnValueSource == GBV_UserBurnValue)?
-                            0 : dfVariant ) );
-            } else {
-                *pdfInsert = ( psInfo->padfBurnValue[iBand] +
-                         ( (psInfo->eBurnValueSource == GBV_UserBurnValue)?
-                            0 : dfVariant ) );
-            }
-        }
-    }
-    else {
-        CPLAssert(false);
+        case GDT_Byte:
+            gvBurnPointBasic<GByte>( psInfo, nY, nX, dfVariant );
+            break;
+        case GDT_Int16:
+            gvBurnPointBasic<GInt16>( psInfo, nY, nX, dfVariant );
+            break;
+        case GDT_UInt16:
+            gvBurnPointBasic<GUInt16>( psInfo, nY, nX, dfVariant );
+            break;
+        case GDT_Int32:
+            gvBurnPointBasic<GInt32>( psInfo, nY, nX, dfVariant );
+            break;
+        case GDT_UInt32:
+            gvBurnPointBasic<GUInt32>( psInfo, nY, nX, dfVariant );
+            break;
+        case GDT_Float32:
+            gvBurnPointBasic<float>( psInfo, nY, nX, dfVariant );
+            break;
+        case GDT_Float64:
+            gvBurnPointBasic<double>( psInfo, nY, nX, dfVariant );
+            break;
+        default:
+            CPLAssert(false);
     }
 }
 
@@ -330,7 +356,9 @@ static void GDALCollectRingsFromGeometry(
 static void
 gv_rasterize_one_shape( unsigned char *pabyChunkBuf, int nXOff, int nYOff,
                         int nXSize, int nYSize,
-                        int nBands, GDALDataType eType, int bAllTouched,
+                        int nBands, GDALDataType eType,
+                        int nPixelSpace, GSpacing nLineSpace, GSpacing nBandSpace,
+                        int bAllTouched,
                         OGRGeometry *poShape, double *padfBurnValue,
                         GDALBurnValueSrc eBurnValueSrc,
                         GDALRasterMergeAlg eMergeAlg,
@@ -341,12 +369,28 @@ gv_rasterize_one_shape( unsigned char *pabyChunkBuf, int nXOff, int nYOff,
     if( poShape == nullptr || poShape->IsEmpty() )
         return;
 
+    if(nPixelSpace == 0)
+    {
+        nPixelSpace = GDALGetDataTypeSizeBytes(eType);
+    }
+    if(nLineSpace == 0)
+    {
+        nLineSpace = static_cast<GSpacing>(nXSize) * nPixelSpace;
+    }
+    if(nBandSpace == 0)
+    {
+        nBandSpace = nYSize * nLineSpace;
+    }
+
     GDALRasterizeInfo sInfo;
     sInfo.nXSize = nXSize;
     sInfo.nYSize = nYSize;
     sInfo.nBands = nBands;
     sInfo.pabyChunkBuf = pabyChunkBuf;
     sInfo.eType = eType;
+    sInfo.nPixelSpace = nPixelSpace;
+    sInfo.nLineSpace = nLineSpace;
+    sInfo.nBandSpace = nBandSpace;
     sInfo.padfBurnValue = padfBurnValue;
     sInfo.eBurnValueSource = eBurnValueSrc;
     sInfo.eMergeAlg = eMergeAlg;
@@ -418,7 +462,8 @@ gv_rasterize_one_shape( unsigned char *pabyChunkBuf, int nXOff, int nYOff,
                                           &(aPointX[0]), &(aPointY[0]),
                                           (eBurnValueSrc == GBV_UserBurnValue)?
                                           nullptr : &(aPointVariant[0]),
-                                          gvBurnPoint, &sInfo );
+                                          gvBurnPoint, &sInfo,
+                                          eMergeAlg == GRMA_Add );
           else
               GDALdllImageLine( sInfo.nXSize, nYSize,
                                 static_cast<int>(aPartSize.size()),
@@ -452,7 +497,8 @@ gv_rasterize_one_shape( unsigned char *pabyChunkBuf, int nXOff, int nYOff,
                       static_cast<int>(aPartSize.size()), &(aPartSize[0]),
                       &(aPointX[0]), &(aPointY[0]),
                       nullptr,
-                      gvBurnPoint, &sInfo );
+                      gvBurnPoint, &sInfo,
+                      eMergeAlg == GRMA_Add );
               }
               else
               {
@@ -469,7 +515,8 @@ gv_rasterize_one_shape( unsigned char *pabyChunkBuf, int nXOff, int nYOff,
                       static_cast<int>(aPartSize.size()), &(aPartSize[0]),
                       &(aPointX[0]), &(aPointY[0]),
                       &(aPointVariant[0]),
-                      gvBurnPoint, &sInfo );
+                      gvBurnPoint, &sInfo,
+                      eMergeAlg == GRMA_Add );
               }
           }
       }
@@ -625,6 +672,51 @@ static CPLErr GDALRasterizeOptions( char **papszOptions,
  * @param pProgressArg callback data for progress function.
  *
  * @return CE_None on success or CE_Failure on error.
+ *
+ * <strong>Example</strong><br>
+ * GDALRasterizeGeometries rasterize output to MEM Dataset :<br>
+ * @code
+ *     int nBufXSize      = 1024;
+ *     int nBufYSize      = 1024;
+ *     int nBandCount     = 1;
+ *     GDALDataType eType = GDT_Byte;
+ *     int nDataTypeSize  = GDALGetDataTypeSizeBytes(eType);
+ *
+ *     void* pData = CPLCalloc( nBufXSize*nBufYSize*nBandCount, nDataTypeSize );
+ *     char memdsetpath[1024];
+ *     sprintf(memdsetpath,"MEM:::DATAPOINTER=0x%p,PIXELS=%d,LINES=%d,"
+ *             "BANDS=%d,DATATYPE=%s,PIXELOFFSET=%d,LINEOFFSET=%d",
+ *             pData,nBufXSize,nBufYSize,nBandCount,GDALGetDataTypeName(eType),
+ *             nBandCount*nDataTypeSize, nBufXSize*nBandCount*nDataTypeSize );
+ *
+ *      // Open Memory Dataset
+ *      GDALDatasetH hMemDset = GDALOpen(memdsetpath, GA_Update);
+ *      // or create it as follows
+ *      // GDALDriverH hMemDriver = GDALGetDriverByName("MEM");
+ *      // GDALDatasetH hMemDset = GDALCreate(hMemDriver, "", nBufXSize, nBufYSize, nBandCount, eType, NULL);
+ *
+ *      double adfGeoTransform[6];
+ *      // Assign GeoTransform parameters,Omitted here.
+ *
+ *      GDALSetGeoTransform(hMemDset,adfGeoTransform);
+ *      GDALSetProjection(hMemDset,pszProjection); // Can not
+ *      
+ *      // Do something ...
+ *      // Need an array of OGRGeometry objects,The assumption here is pahGeoms
+ *      
+ *      int bandList[3] = { 1, 2, 3};
+ *      std::vector<double> geomBurnValue(nGeomCount*nBandCount,255.0);
+ *      CPLErr err = GDALRasterizeGeometries(hMemDset, nBandCount, bandList,
+ *                              nGeomCount, pahGeoms, pfnTransformer, pTransformArg,
+ *                              geomBurnValue.data(), papszOptions,
+ *                              pfnProgress, pProgressArg);
+ *      if( err != CE_None )
+ *      {
+ *          // Do something ...
+ *      }
+ *      GDALClose(hMemDset);
+ *      CPLFree(pData);
+ *@endcode
  */
 
 CPLErr GDALRasterizeGeometries( GDALDatasetH hDS,
@@ -721,7 +813,7 @@ CPLErr GDALRasterizeGeometries( GDALDatasetH hDS,
     if( eOptim == GRO_Auto )
     {
         eOptim = GRO_Raster;
-        // TODO make more tests with various inputs/outputs to ajust the parameters
+        // TODO make more tests with various inputs/outputs to adjust the parameters
         if( nYBlockSize > 1 && nGeomCount > 10000 && (poBand->GetXSize() * static_cast<long long>(poBand->GetYSize()) / nGeomCount > 50) )
         {
             eOptim = GRO_Vector;
@@ -744,8 +836,7 @@ CPLErr GDALRasterizeGeometries( GDALDatasetH hDS,
 /*      size the less times we need to make a pass through all the      */
 /*      shapes.                                                         */
 /* -------------------------------------------------------------------- */
-        const GDALDataType eType =
-            poBand->GetRasterDataType() == GDT_Byte ? GDT_Byte : GDT_Float64;
+        const GDALDataType eType = GDALGetNonComplexDataType(poBand->GetRasterDataType());
 
         const int nScanlineBytes =
             nBandCount * poDS->GetRasterXSize() * GDALGetDataTypeSizeBytes(eType);
@@ -805,7 +896,9 @@ CPLErr GDALRasterizeGeometries( GDALDatasetH hDS,
             {
                 gv_rasterize_one_shape( pabyChunkBuf, 0, iY,
                                         poDS->GetRasterXSize(), nThisYChunkSize,
-                                        nBandCount, eType, bAllTouched,
+                                        nBandCount, eType,
+                                        0, 0, 0,
+                                        bAllTouched,
                                         reinterpret_cast<OGRGeometry *>(
                                                             pahGeometries[iShape]),
                                         padfGeomBurnValue + iShape*nBandCount,
@@ -943,7 +1036,9 @@ CPLErr GDALRasterizeGeometries( GDALDatasetH hDS,
 
                     gv_rasterize_one_shape( pabyChunkBuf, xB * nXBlockSize, yB * nYBlockSize,
                                             nThisXChunkSize, nThisYChunkSize,
-                                            nBandCount, eType, bAllTouched,
+                                            nBandCount, eType,
+                                            0, 0, 0,
+                                            bAllTouched,
                                             reinterpret_cast<OGRGeometry *>(pahGeometries[iShape]),
                                             padfGeomBurnValue + iShape*nBandCount,
                                             eBurnValueSource, eMergeAlg,
@@ -1100,8 +1195,7 @@ CPLErr GDALRasterizeLayers( GDALDatasetH hDS,
     const char  *pszYChunkSize =
         CSLFetchNameValue( papszOptions, "CHUNKYSIZE" );
 
-    const GDALDataType eType =
-        poBand->GetRasterDataType() == GDT_Byte ? GDT_Byte : GDT_Float64;
+    const GDALDataType eType = poBand->GetRasterDataType();
 
     const int nScanlineBytes =
         nBandCount * poDS->GetRasterXSize() * GDALGetDataTypeSizeBytes(eType);
@@ -1302,7 +1396,7 @@ CPLErr GDALRasterizeLayers( GDALDatasetH hDS,
                 gv_rasterize_one_shape( pabyChunkBuf, 0, iY,
                                         poDS->GetRasterXSize(),
                                         nThisYChunkSize,
-                                        nBandCount, eType, bAllTouched, poGeom,
+                                        nBandCount, eType, 0, 0, 0, bAllTouched, poGeom,
                                         padfBurnValues, eBurnValueSource,
                                         eMergeAlg,
                                         pfnTransformer, pTransformArg );
@@ -1380,9 +1474,7 @@ CPLErr GDALRasterizeLayers( GDALDatasetH hDS,
  * needs to transform the geometry locations into pixel/line coordinates
  * of the target raster.
  *
- * The output raster may be of any GDAL supported datatype, though currently
- * internally the burning is done either as GDT_Byte or GDT_Float32.  This
- * may be improved in the future.
+ * The output raster may be of any GDAL supported datatype(non complex).
  *
  * @param pData pointer to the output data array.
  *
@@ -1458,15 +1550,27 @@ CPLErr GDALRasterizeLayersBuf( void *pData, int nBufXSize, int nBufYSize,
                                void *pProgressArg )
 
 {
+/* -------------------------------------------------------------------- */
+/*           check eType, Avoid not supporting data types               */
+/* -------------------------------------------------------------------- */
+    if( GDALDataTypeIsComplex(eBufType) ||
+       eBufType <= GDT_Unknown || eBufType >= GDT_TypeCount )
+    {
+        CPLError(CE_Failure, CPLE_NotSupported,
+            "GDALRasterizeLayersBuf(): unsupported data type of eBufType");
+        return CE_Failure;
+    }
+
 /* -------------------------------------------------------------------- */
 /*      If pixel and line spaceing are defaulted assign reasonable      */
 /*      value assuming a packed buffer.                                 */
 /* -------------------------------------------------------------------- */
-    if( nPixelSpace != 0 )
+    int nTypeSizeBytes = GDALGetDataTypeSizeBytes( eBufType );
+    if( nPixelSpace == 0 )
     {
-        nPixelSpace = GDALGetDataTypeSizeBytes( eBufType );
+        nPixelSpace = nTypeSizeBytes;
     }
-    if( nPixelSpace != GDALGetDataTypeSizeBytes( eBufType ) )
+    if( nPixelSpace < nTypeSizeBytes )
     {
         CPLError(CE_Failure, CPLE_NotSupported,
                  "GDALRasterizeLayersBuf(): unsupported value of nPixelSpace");
@@ -1477,7 +1581,7 @@ CPLErr GDALRasterizeLayersBuf( void *pData, int nBufXSize, int nBufYSize,
     {
         nLineSpace = nPixelSpace * nBufXSize;
     }
-    if( nLineSpace != nPixelSpace * nBufXSize )
+    if( nLineSpace < nPixelSpace * nBufXSize )
     {
         CPLError(CE_Failure, CPLE_NotSupported,
                  "GDALRasterizeLayersBuf(): unsupported value of nLineSpace");
@@ -1599,7 +1703,8 @@ CPLErr GDALRasterizeLayersBuf( void *pData, int nBufXSize, int nBufYSize,
 
                 gv_rasterize_one_shape( static_cast<unsigned char *>(pData), 0, 0,
                                         nBufXSize, nBufYSize,
-                                        1, eBufType, bAllTouched, poGeom,
+                                        1, eBufType,
+                                        nPixelSpace, nLineSpace, 0, bAllTouched, poGeom,
                                         &dfBurnValue, eBurnValueSource,
                                         eMergeAlg,
                                         pfnTransformer, pTransformArg );
