@@ -45,7 +45,7 @@
 #include "cpl_vsi.h"
 #include "gdal.h"
 
-CPL_CVSID("$Id: rasterfill.cpp 659a9911890ce155f5b822462d1bc7804e642644 2018-08-07 22:13:41 +0200 Even Rouault $")
+CPL_CVSID("$Id: rasterfill.cpp c23515b39f088883adf93428c484747e4dcf3a56 2019-02-04 15:11:12 +0100 Even Rouault $")
 
 /************************************************************************/
 /*                           GDALFilterLine()                           */
@@ -342,24 +342,27 @@ GDALMultiFilter( GDALRasterBandH hTargetBand,
 /*      existing closest point.                                         */
 /************************************************************************/
 
-// TODO(schwehr): Convert to an inline function.
-#define QUAD_CHECK(quad_dist, quad_value,                               \
-target_x, target_y, origin_x, origin_y, target_value )                  \
-                                                                        \
-if( quad_value != nNoDataVal )                                          \
-{                                                                       \
-    const double dfDx =                                                 \
-        static_cast<double>(target_x) - static_cast<double>(origin_x);  \
-    const double dfDy =                                                 \
-        static_cast<double>(target_y) - static_cast<double>(origin_y);  \
-    double dfDistSq = dfDx * dfDx + dfDy * dfDy;                        \
-                                                                        \
-    if( dfDistSq < quad_dist*quad_dist )                                \
-    {                                                                   \
-        CPLAssert( dfDistSq > 0.0 );                                    \
-        quad_dist = sqrt(dfDistSq);                                     \
-        quad_value = target_value;                                      \
-    }                                                                   \
+inline void QUAD_CHECK(double& dfQuadDist, float& fQuadValue,
+                       int target_x, GUInt32 target_y,
+                       int origin_x, int origin_y,
+                       float fTargetValue,
+                       GUInt32 nNoDataVal)
+{
+    if( target_y != nNoDataVal )
+    {
+        const double dfDx =
+            static_cast<double>(target_x) - static_cast<double>(origin_x);
+        const double dfDy =
+            static_cast<double>(target_y) - static_cast<double>(origin_y);
+        double dfDistSq = dfDx * dfDx + dfDy * dfDy;
+
+        if( dfDistSq < dfQuadDist*dfQuadDist )
+        {
+            CPLAssert( dfDistSq > 0.0 );
+            dfQuadDist = sqrt(dfDistSq);
+            fQuadValue = fTargetValue;
+        }
+    }
 }
 
 /************************************************************************/
@@ -397,7 +400,7 @@ if( quad_value != nNoDataVal )                                          \
  * <li>TEMP_FILE_DRIVER=gdal_driver_name. For example MEM.</li>
  * <li>NODATA=value (starting with GDAL 2.4).
  * Source pixels at that value will be ignored by the interpolator. Warning:
- * currently this will not be honored by smothing passes.</li>
+ * currently this will not be honored by smoothing passes.</li>
  * </ul>
  * @param pfnProgress the progress function to report completion.
  * @param pProgressArg callback data for progress function.
@@ -669,6 +672,11 @@ GDALFillNodata( GDALRasterBandH hTargetBand,
         }
     }
 
+    for( int iX = 0; iX < nXSize; iX++ )
+    {
+        panLastY[iX] = nNoDataVal;
+    }
+
 /* ==================================================================== */
 /*      Now we will do collect similar this/last information from       */
 /*      bottom to top and use it in combination with the top to         */
@@ -753,7 +761,7 @@ GDALFillNodata( GDALRasterBandH hTargetBand,
 
             // Step left and right by one pixel searching for the closest
             // target value for each quadrant.
-            for( int iStep = 0; iStep < nThisMaxSearchDist; iStep++ )
+            for( int iStep = 0; iStep <= nThisMaxSearchDist; iStep++ )
             {
                 const int iLeftX = std::max(0, iX - iStep);
                 const int iRightX = std::min(nXSize - 1, iX + iStep);
@@ -761,12 +769,12 @@ GDALFillNodata( GDALRasterBandH hTargetBand,
                 // Top left includes current line.
                 QUAD_CHECK(adfQuadDist[0], fQuadValue[0],
                            iLeftX, panTopDownY[iLeftX], iX, iY,
-                           pafTopDownValue[iLeftX] );
+                           pafTopDownValue[iLeftX], nNoDataVal );
 
                 // Bottom left.
                 QUAD_CHECK(adfQuadDist[1], fQuadValue[1],
                            iLeftX, panLastY[iLeftX], iX, iY,
-                           pafLastValue[iLeftX] );
+                           pafLastValue[iLeftX], nNoDataVal );
 
                 // Top right and bottom right do no include center pixel.
                 if( iStep == 0 )
@@ -775,12 +783,12 @@ GDALFillNodata( GDALRasterBandH hTargetBand,
                 // Top right includes current line.
                 QUAD_CHECK(adfQuadDist[2], fQuadValue[2],
                            iRightX, panTopDownY[iRightX], iX, iY,
-                           pafTopDownValue[iRightX] );
+                           pafTopDownValue[iRightX], nNoDataVal );
 
                 // Bottom right.
                 QUAD_CHECK(adfQuadDist[3], fQuadValue[3],
                            iRightX, panLastY[iRightX], iX, iY,
-                           pafLastValue[iRightX] );
+                           pafLastValue[iRightX], nNoDataVal );
 
                 // Every four steps, recompute maximum distance.
                 if( (iStep & 0x3) == 0 )
