@@ -39,7 +39,7 @@
 #define E00ReadNextLine     GDALE00GRIDReadNextLine
 #define E00ReadRewind       GDALE00GRIDReadRewind
 
-CPL_CVSID("$Id: e00griddataset.cpp 62dd0e1d86b69d7cdf7c8731ee6411615f82d414 2018-04-22 15:08:53 +0200 Even Rouault $")
+CPL_CVSID("$Id: e00griddataset.cpp 8e5eeb35bf76390e3134a4ea7076dab7d478ea0e 2018-11-14 22:55:13 +0100 Even Rouault $")
 
 #undef NULL
 #define NULL nullptr
@@ -111,7 +111,10 @@ class E00GRIDDataset : public GDALPamDataset
     ~E00GRIDDataset() override;
 
     CPLErr GetGeoTransform( double * ) override;
-    const char* GetProjectionRef() override;
+    const char* _GetProjectionRef() override;
+    const OGRSpatialReference* GetSpatialRef() const override {
+        return GetSpatialRefFromOldGetProjectionRef();
+    }
 
     static GDALDataset *Open( GDALOpenInfo * );
     static int          Identify( GDALOpenInfo * );
@@ -717,7 +720,7 @@ const char* E00GRIDDataset::ReadLine()
 /*                         GetProjectionRef()                           */
 /************************************************************************/
 
-const char* E00GRIDDataset::GetProjectionRef()
+const char* E00GRIDDataset::_GetProjectionRef()
 
 {
     ReadMetadata();
@@ -850,7 +853,10 @@ void E00GRIDDataset::ReadMetadata()
                 {
                     break;
                 }
-                papszPrj = CSLAddString(papszPrj, pszLine);
+                if (!EQUAL(pszLine, "~") )
+                {
+                    papszPrj = CSLAddString(papszPrj, pszLine);
+                }
             }
 
             OGRSpatialReference oSRS;
