@@ -27,30 +27,19 @@
  * DEALINGS IN THE SOFTWARE.
  ****************************************************************************/
 
-#define H5_USE_16_API
-
-#ifdef _MSC_VER
-#pragma warning(push)
-// warning C4005: '_HDF5USEDLL_' : macro redefinition.
-#pragma warning(disable : 4005)
-#endif
-
-#include "hdf5.h"
-
-#ifdef _MSC_VER
-#pragma warning(pop)
-#endif
+#include "hdf5_api.h"
 
 #include "cpl_string.h"
 #include "gdal_frmts.h"
 #include "gdal_pam.h"
 #include "gdal_priv.h"
+#include "gh5_convenience.h"
 #include "hdf5dataset.h"
 #include "ogr_spatialref.h"
 
 #include <algorithm>
 
-CPL_CVSID("$Id: hdf5imagedataset.cpp d47e89a5a34d08ec47fd0191efce3076246f5dd7 2018-09-20 00:05:59 +0200 Even Rouault $")
+CPL_CVSID("$Id: hdf5imagedataset.cpp 76569154904662bac0f3ddc4bfc110d9b6c8ec0f 2019-04-17 22:42:01 +0200 Even Rouault $")
 
 // Release 1.6.3 or 1.6.4 changed the type of count in some API functions.
 
@@ -108,9 +97,15 @@ public:
     static GDALDataset  *Open( GDALOpenInfo * );
     static int           Identify( GDALOpenInfo * );
 
-    const char          *GetProjectionRef() override;
+    const char          *_GetProjectionRef() override;
+    const OGRSpatialReference* GetSpatialRef() const override {
+        return GetSpatialRefFromOldGetProjectionRef();
+    }
     virtual int         GetGCPCount() override;
-    virtual const char  *GetGCPProjection() override;
+    virtual const char  *_GetGCPProjection() override;
+        const OGRSpatialReference* GetGCPSpatialRef() const override {
+        return GetGCPSpatialRefFromOldGetGCPProjection();
+    }
     virtual const GDAL_GCP *GetGCPs() override;
     virtual CPLErr GetGeoTransform( double *padfTransform ) override;
 
@@ -246,7 +241,6 @@ class HDF5ImageRasterBand : public GDALPamRasterBand
 
     virtual CPLErr      IReadBlock( int, int, void * ) override;
     virtual double      GetNoDataValue( int * ) override;
-    virtual CPLErr      SetNoDataValue( double ) override;
     // virtual CPLErr IWriteBlock( int, int, void * );
 };
 
@@ -305,6 +299,12 @@ HDF5ImageRasterBand::HDF5ImageRasterBand( HDF5ImageDataset *poDSIn, int nBandIn,
 
         H5Pclose(listid);
     }
+
+    // netCDF convention for nodata
+    bNoDataSet = GH5_FetchAttribute(
+        poDSIn->dataset_id, "_FillValue", dfNoDataValue);
+    if( !bNoDataSet )
+        dfNoDataValue = -9999.0;
 }
 
 /************************************************************************/
@@ -324,18 +324,6 @@ double HDF5ImageRasterBand::GetNoDataValue( int *pbSuccess )
     return GDALPamRasterBand::GetNoDataValue(pbSuccess);
 }
 
-/************************************************************************/
-/*                           SetNoDataValue()                           */
-/************************************************************************/
-CPLErr HDF5ImageRasterBand::SetNoDataValue( double dfNoData )
-
-{
-    bNoDataSet = true;
-    dfNoDataValue = dfNoData;
-
-    return CE_None;
-}
-
 /************************************************************************/
 /*                             IReadBlock()                             */
 /************************************************************************/
@@ -344,10 +332,11 @@ CPLErr HDF5ImageRasterBand::IReadBlock( int nBlockXOff, int nBlockYOff,
 {
     HDF5ImageDataset *poGDS = static_cast<HDF5ImageDataset *>(poDS);
 
+    memset(pImage, 0,
+            static_cast<size_t>(nBlockXSize) * nBlockYSize * GDALGetDataTypeSizeBytes(eDataType));
+
     if( poGDS->eAccess == GA_Update )
     {
-        memset(pImage, 0,
-               nBlockXSize * nBlockYSize * GDALGetDataTypeSize(eDataType) / 8);
         return CE_None;
     }
 
@@ -377,9 +366,6 @@ CPLErr HDF5ImageRasterBand::IReadBlock( int nBlockXOff, int nBlockYOff,
     count[poGDS->GetYIndex()] = nBlockYSize;
     count[poGDS->GetXIndex()] = nBlockXSize;
 
-    const int nSizeOfData = static_cast<int>(H5Tget_size(poGDS->native));
-    memset(pImage, 0, nBlockXSize * nBlockYSize * nSizeOfData);
-
     // Blocksize may not be a multiple of imagesize.
     count[poGDS->GetYIndex()] =
         std::min(hsize_t(nBlockYSize),
@@ -475,7 +461,8 @@ GDALDataset *HDF5ImageDataset::Open( GDALOpenInfo *poOpenInfo )
 
     CPLString osFilename(papszName[1]);
 
-    if( strlen(papszName[1]) == 1 && papszName[3] != nullptr )
+    if( (strlen(papszName[1]) == 1 && papszName[3] != nullptr) ||
+        (STARTS_WITH(papszName[1], "/vsicurl/http") && papszName[3] != nullptr) )
     {
         osFilename += ":";
         osFilename += papszName[2];
@@ -491,16 +478,13 @@ GDALDataset *HDF5ImageDataset::Open( GDALOpenInfo *poOpenInfo )
     CSLDestroy(papszName);
     papszName = nullptr;
 
-    if( !H5Fis_hdf5(osFilename) )
-    {
-        delete poDS;
-        return nullptr;
-    }
-
     poDS->SetPhysicalFilename(osFilename);
 
     // Try opening the dataset.
-    poDS->hHDF5 = H5Fopen(osFilename, H5F_ACC_RDONLY, H5P_DEFAULT);
+    hid_t fapl = H5Pcreate(H5P_FILE_ACCESS);
+    H5Pset_driver(fapl, HDF5GetFileDriver(), nullptr);
+    poDS->hHDF5 = H5Fopen(osFilename, H5F_ACC_RDONLY, fapl);
+    H5Pclose(fapl);
 
     if( poDS->hHDF5 < 0 )
     {
@@ -580,8 +564,6 @@ GDALDataset *HDF5ImageDataset::Open( GDALOpenInfo *poOpenInfo )
             new HDF5ImageRasterBand(poDS, i, poDS->GetDataType(poDS->native));
 
         poDS->SetBand(i, poBand);
-        if( poBand->bNoDataSet )
-            poBand->SetNoDataValue(255);
     }
 
     poDS->CreateProjections();
@@ -595,6 +577,15 @@ GDALDataset *HDF5ImageDataset::Open( GDALOpenInfo *poOpenInfo )
     return poDS;
 }
 
+/************************************************************************/
+/*                   HDF5ImageDatasetDriverUnload()                     */
+/************************************************************************/
+
+static void HDF5ImageDatasetDriverUnload(GDALDriver*)
+{
+    HDF5UnloadFileDriver();
+}
+
 /************************************************************************/
 /*                        GDALRegister_HDF5Image()                      */
 /************************************************************************/
@@ -613,9 +604,11 @@ void GDALRegister_HDF5Image()
     poDriver->SetMetadataItem(GDAL_DCAP_RASTER, "YES");
     poDriver->SetMetadataItem(GDAL_DMD_LONGNAME, "HDF5 Dataset");
     poDriver->SetMetadataItem(GDAL_DMD_HELPTOPIC, "frmt_hdf5.html");
+    poDriver->SetMetadataItem( GDAL_DCAP_VIRTUALIO, "YES" );
 
     poDriver->pfnOpen = HDF5ImageDataset::Open;
     poDriver->pfnIdentify = HDF5ImageDataset::Identify;
+    poDriver->pfnUnloadDriver = HDF5ImageDatasetDriverUnload;
 
     GetGDALDriverManager()->RegisterDriver(poDriver);
 }
@@ -645,11 +638,13 @@ CPLErr HDF5ImageDataset::CreateODIMH5Projection()
         pszUR_lon == nullptr || pszUR_lat == nullptr )
         return CE_Failure;
 
+    oSRS.SetAxisMappingStrategy(OAMS_TRADITIONAL_GIS_ORDER);
     if( oSRS.importFromProj4(pszProj4String) != OGRERR_NONE )
         return CE_Failure;
 
     OGRSpatialReference oSRSWGS84;
     oSRSWGS84.SetWellKnownGeogCS("WGS84");
+    oSRSWGS84.SetAxisMappingStrategy(OAMS_TRADITIONAL_GIS_ORDER);
 
     OGRCoordinateTransformation *poCT =
         OGRCreateCoordinateTransformation(&oSRSWGS84, &oSRS);
@@ -748,7 +743,9 @@ CPLErr HDF5ImageDataset::CreateProjections()
 
         // The Latitude and Longitude arrays must have a rank of 2 to retrieve
         // GCPs.
-        if( poH5Objects->nRank != 2 )
+        if( poH5Objects->nRank != 2 ||
+            poH5Objects->paDims[0] != static_cast<size_t>(nRasterYSize) ||
+            poH5Objects->paDims[1] != static_cast<size_t>(nRasterXSize) )
         {
             return CE_None;
         }
@@ -758,6 +755,17 @@ CPLErr HDF5ImageDataset::CreateProjections()
         // LatitudeDataspaceID = H5Dget_space(dataset_id);
 
         poH5Objects = HDF5FindDatasetObjects(poH5RootGroup, "Longitude");
+        // GCPs.
+        if( poH5Objects == nullptr ||
+            poH5Objects->nRank != 2 ||
+            poH5Objects->paDims[0] != static_cast<size_t>(nRasterYSize) ||
+            poH5Objects->paDims[1] != static_cast<size_t>(nRasterXSize) )
+        {
+            if( LatitudeDatasetID > 0 )
+                H5Dclose(LatitudeDatasetID);
+            return CE_None;
+        }
+
         const hid_t LongitudeDatasetID = H5Dopen(hHDF5, poH5Objects->pszPath);
         // LongitudeDataspaceID = H5Dget_space(dataset_id);
 
@@ -770,6 +778,15 @@ CPLErr HDF5ImageDataset::CreateProjections()
             memset(Latitude, 0, nRasterXSize * nRasterYSize * sizeof(float));
             memset(Longitude, 0, nRasterXSize * nRasterYSize * sizeof(float));
 
+            // netCDF convention for nodata
+            double dfLatNoData = 0;
+            bool bHasLatNoData = GH5_FetchAttribute(
+                LatitudeDatasetID, "_FillValue", dfLatNoData);
+
+            double dfLongNoData = 0;
+            bool bHasLongNoData = GH5_FetchAttribute(
+                LongitudeDatasetID, "_FillValue", dfLongNoData);
+
             H5Dread(LatitudeDatasetID, H5T_NATIVE_FLOAT, H5S_ALL, H5S_ALL,
                     H5P_DEFAULT, Latitude);
 
@@ -778,19 +795,10 @@ CPLErr HDF5ImageDataset::CreateProjections()
 
             oSRS.SetWellKnownGeogCS("WGS84");
             CPLFree(pszProjection);
+            pszProjection = nullptr;
             CPLFree(pszGCPProjection);
-            oSRS.exportToWkt(&pszProjection);
             oSRS.exportToWkt(&pszGCPProjection);
 
-            // Fill the GCPs list.
-
-            nGCPCount = (nRasterYSize / nDeltaLat) * (nRasterXSize / nDeltaLon);
-
-            pasGCPList =
-                static_cast<GDAL_GCP *>(CPLCalloc(nGCPCount, sizeof(GDAL_GCP)));
-
-            GDALInitGCPs(nGCPCount, pasGCPList);
-
             const int nYLimit =
                 (static_cast<int>(nRasterYSize) / nDeltaLat) * nDeltaLat;
             const int nXLimit =
@@ -804,19 +812,32 @@ CPLErr HDF5ImageDataset::CreateProjections()
             bool bHasLonNearMinus180 = false;
             bool bHasLonNearPlus180 = false;
             bool bHasLonNearZero = false;
+            nGCPCount = 0;
             for( int j = 0; j < nYLimit; j += nDeltaLat )
             {
                 for( int i = 0; i < nXLimit; i += nDeltaLon )
                 {
                     const int iGCP = j * nRasterXSize + i;
+                    if( (bHasLatNoData && static_cast<float>(dfLatNoData) == Latitude[iGCP]) ||
+                        (bHasLongNoData && static_cast<float>(dfLongNoData) == Longitude[iGCP]) )
+                        continue;
                     if( Longitude[iGCP] > 170 && Longitude[iGCP] <= 180 )
                         bHasLonNearPlus180 = true;
                     if( Longitude[iGCP] < -170 && Longitude[iGCP] >= -180 )
                         bHasLonNearMinus180 = true;
                     if( fabs(Longitude[iGCP]) < 90 )
                         bHasLonNearZero = true;
+                    nGCPCount ++;
                 }
             }
+
+            // Fill the GCPs list.
+
+            pasGCPList =
+                static_cast<GDAL_GCP *>(CPLCalloc(nGCPCount, sizeof(GDAL_GCP)));
+
+            GDALInitGCPs(nGCPCount, pasGCPList);
+
             const char *pszShiftGCP =
                 CPLGetConfigOption("HDF5_SHIFT_GCPX_BY_180", nullptr);
             const bool bAdd180 =
@@ -830,6 +851,9 @@ CPLErr HDF5ImageDataset::CreateProjections()
                 for( int i = 0; i < nXLimit; i += nDeltaLon )
                 {
                     const int iGCP = j * nRasterXSize + i;
+                    if( (bHasLatNoData && static_cast<float>(dfLatNoData) == Latitude[iGCP]) ||
+                        (bHasLongNoData && static_cast<float>(dfLongNoData) == Longitude[iGCP]) )
+                        continue;
                     pasGCPList[k].dfGCPX = static_cast<double>(Longitude[iGCP]);
                     if( bAdd180 )
                         pasGCPList[k].dfGCPX += 180.0;
@@ -860,13 +884,13 @@ CPLErr HDF5ImageDataset::CreateProjections()
 /*                          GetProjectionRef()                          */
 /************************************************************************/
 
-const char *HDF5ImageDataset::GetProjectionRef()
+const char *HDF5ImageDataset::_GetProjectionRef()
 
 {
     if( pszProjection )
         return pszProjection;
 
-    return GDALPamDataset::GetProjectionRef();
+    return GDALPamDataset::_GetProjectionRef();
 }
 
 /************************************************************************/
@@ -886,13 +910,13 @@ int HDF5ImageDataset::GetGCPCount()
 /*                          GetGCPProjection()                          */
 /************************************************************************/
 
-const char *HDF5ImageDataset::GetGCPProjection()
+const char *HDF5ImageDataset::_GetGCPProjection()
 
 {
     if( nGCPCount > 0 )
         return pszGCPProjection;
 
-    return GDALPamDataset::GetGCPProjection();
+    return GDALPamDataset::_GetGCPProjection();
 }
 
 /************************************************************************/
