@@ -32,7 +32,7 @@
 #include "gdal_frmts.h"
 #include "gdal_pam.h"
 
-CPL_CVSID("$Id: jaxapalsardataset.cpp a542b2797f15f2ed694cfcee9ff17d86b339dfee 2018-04-02 00:24:03 +0200 Even Rouault $")
+CPL_CVSID("$Id: jaxapalsardataset.cpp 88eda08930b6dafb9ea1374ba19e0b1cf5ded3d3 2018-08-11 20:16:37 +0200 Even Rouault $")
 
 #if defined(WIN32)
 #define SEP_STRING "\\"
@@ -470,6 +470,17 @@ void PALSARJaxaDataset::ReadMetadata( PALSARJaxaDataset *poDS, VSILFILE *fp ) {
 /*                              Identify()                              */
 /************************************************************************/
 
+static void ReadWord(VSILFILE* fp, int* pVal)
+{
+    READ_WORD(fp, *pVal);
+}
+
+static void ReadByte(VSILFILE* fp, int* pVal)
+{
+    READ_BYTE(fp, *pVal);
+}
+
+
 int PALSARJaxaDataset::Identify( GDALOpenInfo *poOpenInfo ) {
     if ( poOpenInfo->nHeaderBytes < 360 || poOpenInfo->fpL == nullptr )
         return 0;
@@ -491,12 +502,12 @@ int PALSARJaxaDataset::Identify( GDALOpenInfo *poOpenInfo ) {
 
     VSIFSeekL(poOpenInfo->fpL, 0, SEEK_SET);
 
-    READ_WORD(poOpenInfo->fpL, nRecordSeq);
-    READ_BYTE(poOpenInfo->fpL, nRecordSubtype);
-    READ_BYTE(poOpenInfo->fpL, nRecordType);
-    READ_BYTE(poOpenInfo->fpL, nSecondSubtype);
-    READ_BYTE(poOpenInfo->fpL, nThirdSubtype);
-    READ_WORD(poOpenInfo->fpL, nLengthRecord);
+    ReadWord(poOpenInfo->fpL, &nRecordSeq);
+    ReadByte(poOpenInfo->fpL, &nRecordSubtype);
+    ReadByte(poOpenInfo->fpL, &nRecordType);
+    ReadByte(poOpenInfo->fpL, &nSecondSubtype);
+    ReadByte(poOpenInfo->fpL, &nThirdSubtype);
+    ReadWord(poOpenInfo->fpL, &nLengthRecord);
 
     VSIFSeekL(poOpenInfo->fpL, 0, SEEK_SET);
 
