@@ -31,9 +31,11 @@
 #define SQUARE(x) ((x)*(x))
 #define EPSILON 1e-5
 
-CPL_CVSID("$Id: pdfreadvectors.cpp 6574497e5ddfd7c08c094a76756a0ef477cef6a1 2018-04-04 22:15:20 +0200 Even Rouault $")
+CPL_CVSID("$Id: pdfreadvectors.cpp 5b629d218bb9b280261daf8f4d1b22f904729505 2019-10-19 19:57:30 +0200 Even Rouault $")
 
-#if defined(HAVE_POPPLER) || defined(HAVE_PODOFO) || defined(HAVE_PDFIUM)
+#ifdef HAVE_PDF_READ_SUPPORT
+
+constexpr int BEZIER_STEPS = 10;
 
 /************************************************************************/
 /*                        OpenVectorLayers()                            */
@@ -245,7 +247,7 @@ void PDFDataset::ExploreTree(GDALPDFObject* poObj,
     if (nRecLevel == 16)
         return;
 
-    std::pair<int,int> oObjPair( poObj->GetRefNum(), poObj->GetRefGen() );
+    std::pair<int,int> oObjPair( poObj->GetRefNum().toInt(), poObj->GetRefGen() );
     if( aoSetAlreadyVisited.find( oObjPair ) != aoSetAlreadyVisited.end() )
         return;
     aoSetAlreadyVisited.insert( oObjPair );
@@ -277,6 +279,7 @@ void PDFDataset::ExploreTree(GDALPDFObject* poObj,
     {
         GDALPDFArray* poArray = poK->GetArray();
         if (poArray->GetLength() > 0 &&
+            poArray->Get(0) &&
             poArray->Get(0)->GetType() == PDFObjectType_Dictionary &&
             poArray->Get(0)->GetDictionary()->Get("K") != nullptr &&
             poArray->Get(0)->GetDictionary()->Get("K")->GetType() == PDFObjectType_Int)
@@ -292,17 +295,12 @@ void PDFDataset::ExploreTree(GDALPDFObject* poObj,
                     osLayerName = CPLSPrintf("Layer%d", nLayers + 1);
             }
 
-            const char* l_pszWKT = GetProjectionRef();
-            OGRSpatialReference* poSRS = nullptr;
-            if (l_pszWKT && l_pszWKT[0] != '\0')
-            {
-                poSRS = new OGRSpatialReference();
-                poSRS->importFromWkt(l_pszWKT);
-            }
-
+            auto poSRSOri = GetSpatialRef();
+            OGRSpatialReference* poSRS = poSRSOri ? poSRSOri->Clone() : nullptr;
             OGRPDFLayer* poLayer =
                 new OGRPDFLayer(this, osLayerName.c_str(), poSRS, wkbUnknown);
-            delete poSRS;
+            if( poSRS )
+                poSRS->Release();
 
             poLayer->Fill(poArray);
 
@@ -314,8 +312,14 @@ void PDFDataset::ExploreTree(GDALPDFObject* poObj,
         else
         {
             for(int i=0;i<poArray->GetLength();i++)
-                ExploreTree(poArray->Get(i), aoSetAlreadyVisited,
-                            nRecLevel + 1);
+            {
+                auto poSubObj = poArray->Get(i);
+                if (poSubObj )
+                {
+                    ExploreTree(poSubObj, aoSetAlreadyVisited,
+                                nRecLevel + 1);
+                }
+            }
         }
     }
     else if (poK->GetType() == PDFObjectType_Dictionary)
@@ -432,16 +436,16 @@ void PDFDataset::PDFCoordsToSRSCoords(double x, double y,
 
 static OGRPoint* PDFGetCircleCenter(OGRLineString* poLS)
 {
-    if (poLS == nullptr || poLS->getNumPoints() != 5)
+    if (poLS == nullptr || poLS->getNumPoints() != 1 + 4 * BEZIER_STEPS)
         return nullptr;
 
-    if (poLS->getY(0) == poLS->getY(2) &&
-        poLS->getX(1) == poLS->getX(3) &&
-        fabs((poLS->getX(0) + poLS->getX(2)) / 2 - poLS->getX(1)) < EPSILON &&
-        fabs((poLS->getY(1) + poLS->getY(3)) / 2 - poLS->getY(0)) < EPSILON)
+    if (poLS->getY(0 * BEZIER_STEPS) == poLS->getY(2 * BEZIER_STEPS) &&
+        poLS->getX(1 * BEZIER_STEPS) == poLS->getX(3 * BEZIER_STEPS) &&
+        fabs((poLS->getX(0 * BEZIER_STEPS) + poLS->getX(2 * BEZIER_STEPS)) / 2 - poLS->getX(1 * BEZIER_STEPS)) < EPSILON &&
+        fabs((poLS->getY(1 * BEZIER_STEPS) + poLS->getY(3 * BEZIER_STEPS)) / 2 - poLS->getY(0 * BEZIER_STEPS)) < EPSILON)
     {
-        return new OGRPoint((poLS->getX(0) + poLS->getX(2)) / 2,
-                            (poLS->getY(1) + poLS->getY(3)) / 2);
+        return new OGRPoint((poLS->getX(0 * BEZIER_STEPS) + poLS->getX(2 * BEZIER_STEPS)) / 2,
+                            (poLS->getY(1 * BEZIER_STEPS) + poLS->getY(3 * BEZIER_STEPS)) / 2);
     }
     return nullptr;
 }
@@ -562,6 +566,42 @@ int PDFDataset::UnstackTokens(const char* pszToken,
     return TRUE;
 }
 
+/************************************************************************/
+/*                           AddBezierCurve()                           */
+/************************************************************************/
+
+static void AddBezierCurve(std::vector<double>& oCoords,
+                           const double* x0_y0,
+                           const double* x1_y1,
+                           const double* x2_y2,
+                           const double* x3_y3)
+{
+    double x0 = x0_y0[0];
+    double y0 = x0_y0[1];
+    double x1 = x1_y1[0];
+    double y1 = x1_y1[1];
+    double x2 = x2_y2[0];
+    double y2 = x2_y2[1];
+    double x3 = x3_y3[0];
+    double y3 = x3_y3[1];
+    for( int i = 1; i < BEZIER_STEPS; i++ )
+    {
+        const double t = static_cast<double>(i) / BEZIER_STEPS;
+        const double t2 = t * t;
+        const double t3 = t2 * t;
+        const double oneMinust = 1 - t;
+        const double oneMinust2 = oneMinust * oneMinust;
+        const double oneMinust3 = oneMinust2 * oneMinust;
+        const double three_t_oneMinust = 3 * t * oneMinust;
+        const double x = oneMinust3 * x0 + three_t_oneMinust * (oneMinust * x1 + t * x2) + t3 * x3;
+        const double y = oneMinust3 * y0 + three_t_oneMinust * (oneMinust * y1 + t * y2) + t3 * y3;
+        oCoords.push_back(x);
+        oCoords.push_back(y);
+    }
+    oCoords.push_back(x3);
+    oCoords.push_back(y3);
+}
+
 /************************************************************************/
 /*                           ParseContent()                             */
 /************************************************************************/
@@ -662,7 +702,7 @@ OGRGeometry* PDFDataset::ParseContent(const char* pszContent,
         {
             nArrayLevel ++;
         }
-        else if (!bInString && nArrayLevel && nTokenSize == 0 && ch == ']')
+        else if (!bInString && nArrayLevel && ch == ']')
         {
             nArrayLevel --;
         }
@@ -688,6 +728,54 @@ OGRGeometry* PDFDataset::ParseContent(const char* pszContent,
                 bPushToken = TRUE;
             }
         }
+        else if( bInString && ch == '\\' )
+        {
+            const auto nextCh = pszContent[1];
+            if( nextCh == 'n' )
+            {
+                ADD_CHAR(szToken, '\n');
+                pszContent ++;
+            }
+            else if( nextCh == 'r' )
+            {
+                ADD_CHAR(szToken, '\r');
+                pszContent ++;
+            }
+            else if( nextCh == 't' )
+            {
+                ADD_CHAR(szToken, '\t');
+                pszContent ++;
+            }
+            else if( nextCh == 'b' )
+            {
+                ADD_CHAR(szToken, '\b');
+                pszContent ++;
+            }
+            else if( nextCh == '(' || nextCh == ')' || nextCh == '\\' )
+            {
+                ADD_CHAR(szToken, nextCh);
+                pszContent ++;
+            }
+            else if( nextCh >= '0' && nextCh <= '7' &&
+                     pszContent[2] >= '0' && pszContent[2] <= '7' &&
+                     pszContent[3] >= '0' && pszContent[3] <= '7' )
+            {
+                ADD_CHAR(szToken,
+                         ((nextCh - '\0') * 64 + (pszContent[2] - '\0') * 8 + pszContent[3] - '\0'));
+                pszContent += 3;
+            }
+            else if( nextCh == '\n' )
+            {
+                if( pszContent[2] == '\r' )
+                    pszContent += 2;
+                else
+                    pszContent ++;
+            }
+            else if( nextCh == '\r' )
+            {
+                pszContent ++;
+            }
+        }
         else if (ch == '<' && pszContent[1] == '<' && nTokenSize == 0)
         {
             int nDictDepth = 0;
@@ -726,7 +814,12 @@ OGRGeometry* PDFDataset::ParseContent(const char* pszContent,
         }
         else
         {
-            ADD_CHAR(szToken, ch);
+            // Do not create too long tokens in arrays, that we will ignore
+            // anyway
+            if( nArrayLevel == 0 || nTokenSize == 0 )
+            {
+                ADD_CHAR(szToken, ch);
+            }
         }
 
         pszContent ++;
@@ -948,11 +1041,16 @@ OGRGeometry* PDFDataset::ParseContent(const char* pszContent,
                         return nullptr;
                     }
 
+                    oGS.ApplyMatrix(adfCoords + 0);
+                    oGS.ApplyMatrix(adfCoords + 2);
                     oGS.ApplyMatrix(adfCoords + 4);
-                    oCoords.push_back(adfCoords[4]);
-                    oCoords.push_back(adfCoords[5]);
+                    AddBezierCurve(oCoords,
+                                   oCoords.empty() ? &adfCoords[0] : &oCoords[oCoords.size()-2],
+                                   &adfCoords[0],
+                                   &adfCoords[2],
+                                   &adfCoords[4]);
                 }
-                else if (EQUAL1(szToken, "v") || EQUAL1(szToken, "y")) /* Bezier curve */
+                else if (EQUAL1(szToken, "v")) /* Bezier curve */
                 {
                     double adfCoords[4];
                     if (!UnstackTokens(szToken, 4, aszTokenStack, nTokenStackSize, adfCoords))
@@ -961,9 +1059,30 @@ OGRGeometry* PDFDataset::ParseContent(const char* pszContent,
                         return nullptr;
                     }
 
+                    oGS.ApplyMatrix(adfCoords + 0);
                     oGS.ApplyMatrix(adfCoords + 2);
-                    oCoords.push_back(adfCoords[2]);
-                    oCoords.push_back(adfCoords[3]);
+                    AddBezierCurve(oCoords,
+                                   oCoords.empty() ? &adfCoords[0] : &oCoords[oCoords.size()-2],
+                                   oCoords.empty() ? &adfCoords[0] : &oCoords[oCoords.size()-2],
+                                   &adfCoords[0],
+                                   &adfCoords[2]);
+                }
+                else if (EQUAL1(szToken, "y")) /* Bezier curve */
+                {
+                    double adfCoords[4];
+                    if (!UnstackTokens(szToken, 4, aszTokenStack, nTokenStackSize, adfCoords))
+                    {
+                        CPLDebug("PDF", "Should not happen at line %d", __LINE__);
+                        return nullptr;
+                    }
+
+                    oGS.ApplyMatrix(adfCoords + 0);
+                    oGS.ApplyMatrix(adfCoords + 2);
+                    AddBezierCurve(oCoords,
+                                   oCoords.empty() ? &adfCoords[0] : &oCoords[oCoords.size()-2],
+                                   &adfCoords[0],
+                                   &adfCoords[2],
+                                   &adfCoords[2]);
                 }
                 else if (EQUAL2(szToken, "re")) /* Rectangle */
                 {
@@ -1263,7 +1382,7 @@ OGRGeometry* PDFDataset::BuildGeometry(std::vector<double>& oCoords,
 
         // Recognize points as written by GDAL (ogr-sym-2 : circle (not filled))
         OGRGeometry* poCenter = nullptr;
-        if (poCenter == nullptr && poLS != nullptr && poLS->getNumPoints() == 5)
+        if (poCenter == nullptr && poLS != nullptr && poLS->getNumPoints() == 1 + BEZIER_STEPS * 4 )
         {
             poCenter = PDFGetCircleCenter(poLS);
         }
@@ -1330,6 +1449,15 @@ OGRGeometry* PDFDataset::BuildGeometry(std::vector<double>& oCoords,
         {
             if (oCoords[i] == NEW_SUBPATH && oCoords[i+1] == NEW_SUBPATH)
             {
+                if (poLS && poLS->getNumPoints() >= 3)
+                {
+                    OGRPolygon* poPoly =  new OGRPolygon();
+                    poPoly->addRingDirectly(poLS);
+                    poLS = nullptr;
+
+                    papoPoly = (OGRGeometry**) CPLRealloc(papoPoly, (nPolys + 1) * sizeof(OGRGeometry*));
+                    papoPoly[nPolys ++] = poPoly;
+                }
                 delete poLS;
                 poLS = new OGRLinearRing();
             }
@@ -1342,16 +1470,21 @@ OGRGeometry* PDFDataset::BuildGeometry(std::vector<double>& oCoords,
 
                     OGRPoint* poCenter = nullptr;
 
+                   if (nPolys == 0 &&
+                        poLS &&
+                        poLS->getNumPoints() == 1 + BEZIER_STEPS * 4 )
+                   {
+                        // Recognize points as written by GDAL (ogr-sym-3 : circle (filled))
+                        poCenter = PDFGetCircleCenter(poLS);
+                   }
+
                     if (nPolys == 0 &&
+                        poCenter == nullptr &&
                         poLS &&
                         poLS->getNumPoints() == 5)
                     {
-                        // Recognize points as written by GDAL (ogr-sym-3 : circle (filled))
-                        poCenter = PDFGetCircleCenter(poLS);
-
                         // Recognize points as written by GDAL (ogr-sym-5: square (filled))
-                        if (poCenter == nullptr)
-                            poCenter = PDFGetSquareCenter(poLS);
+                        poCenter = PDFGetSquareCenter(poLS);
 
                         /* ESRI points */
                         if (poCenter == nullptr &&
@@ -1472,7 +1605,13 @@ void PDFDataset::ExploreContents(GDALPDFObject* poObj,
     {
         GDALPDFArray* poArray = poObj->GetArray();
         for(int i=0;i<poArray->GetLength();i++)
-            ExploreContents(poArray->Get(i), poResources);
+        {
+            GDALPDFObject* poSubObj = poArray->Get(i);
+            if( poSubObj )
+            {
+                ExploreContents(poSubObj, poResources);
+            }
+        }
     }
 
     if (poObj->GetType() != PDFObjectType_Dictionary)
@@ -1548,7 +1687,7 @@ void PDFDataset::ExploreContentsNonStructuredInternal(GDALPDFObject* poContents,
         for(int i=0;i<poArray->GetLength();i++)
         {
             GDALPDFObject* poObj = poArray->Get(i);
-            if( poObj->GetType() != PDFObjectType_Dictionary)
+            if( poObj == nullptr || poObj->GetType() != PDFObjectType_Dictionary)
                 break;
             GDALPDFStream* poStream = poObj->GetStream();
             if (!poStream)
@@ -1623,17 +1762,12 @@ void PDFDataset::ExploreContentsNonStructured(GDALPDFObject* poContents,
                 OGRPDFLayer* poLayer = (OGRPDFLayer*) GetLayerByName(osSanitizedName.c_str());
                 if (poLayer == nullptr)
                 {
-                    const char* l_pszWKT = GetProjectionRef();
-                    OGRSpatialReference* poSRS = nullptr;
-                    if (l_pszWKT && l_pszWKT[0] != '\0')
-                    {
-                        poSRS = new OGRSpatialReference();
-                        poSRS->importFromWkt(l_pszWKT);
-                    }
-
+                    auto poSRSOri = GetSpatialRef();
+                    OGRSpatialReference* poSRS = poSRSOri ? poSRSOri->Clone() : nullptr;
                     poLayer =
                         new OGRPDFLayer(this, osSanitizedName.c_str(), poSRS, wkbUnknown);
-                    delete poSRS;
+                    if( poSRS )
+                        poSRS->Release();
 
                     papoLayers = (OGRLayer**)
                         CPLRealloc(papoLayers, (nLayers + 1) * sizeof(OGRLayer*));
@@ -1656,11 +1790,11 @@ void PDFDataset::ExploreContentsNonStructured(GDALPDFObject* poContents,
             {
                 const char* pszKey = oIter->first.c_str();
                 GDALPDFObject* poObj = oIter->second;
-                if( poObj->GetRefNum() != 0 )
+                if( poObj->GetRefNum().toBool() )
                 {
                     std::map< std::pair<int, int>, OGRPDFLayer *>::iterator
                         oIterNumGenToLayer = oMapNumGenToLayer.find(
-                            std::pair<int,int>(poObj->GetRefNum(), poObj->GetRefGen()) );
+                            std::pair<int,int>(poObj->GetRefNum().toInt(), poObj->GetRefGen()) );
                     if( oIterNumGenToLayer != oMapNumGenToLayer.end() )
                     {
                         oMapPropertyToLayer[pszKey] = oIterNumGenToLayer->second;
@@ -1713,4 +1847,4 @@ void PDFDataset::ExploreContentsNonStructured(GDALPDFObject* poContents,
     }
 }
 
-#endif /* defined(HAVE_POPPLER) || defined(HAVE_PODOFO) || defined(HAVE_PDFIUM) */
+#endif /* HAVE_PDF_READ_SUPPORT */
