@@ -30,6 +30,7 @@
 #include "cpl_port.h"
 #include "gdal_priv.h"
 
+#include <cassert>
 #include <climits>
 #include <cstddef>
 #include <new>
@@ -45,7 +46,7 @@ constexpr int SUBBLOCK_SIZE = 64;
 #define TO_SUBBLOCK(x) ((x) >> 6)
 #define WITHIN_SUBBLOCK(x) ((x) & 0x3f)
 
-CPL_CVSID("$Id: gdalarraybandblockcache.cpp 0eef25187324a7b0cbff6a57927945a33b28cb55 2018-04-04 21:16:46 +0200 Even Rouault $")
+CPL_CVSID("$Id: gdalarraybandblockcache.cpp e3511844a11959bd81f5956a1e7e2de96900e7ac 2019-04-25 11:53:15 +0200 Even Rouault $")
 
 /* ******************************************************************** */
 /*                        GDALArrayBandBlockCache                       */
@@ -53,14 +54,18 @@ CPL_CVSID("$Id: gdalarraybandblockcache.cpp 0eef25187324a7b0cbff6a57927945a33b28
 
 class GDALArrayBandBlockCache final : public GDALAbstractBandBlockCache
 {
-    bool              bSubBlockingActive;
-    int               nSubBlocksPerRow;
-    int               nSubBlocksPerColumn;
-    union
+    bool              bSubBlockingActive = false;
+    int               nSubBlocksPerRow = 0;
+    int               nSubBlocksPerColumn = 0;
+    union u
     {
         GDALRasterBlock **papoBlocks;
         GDALRasterBlock ***papapoBlocks;
-    } u;
+
+        u(): papoBlocks(nullptr) {}
+    } u{};
+
+    CPL_DISALLOW_COPY_ASSIGN(GDALArrayBandBlockCache)
 
   public:
     explicit GDALArrayBandBlockCache( GDALRasterBand* poBand );
@@ -91,12 +96,8 @@ GDALAbstractBandBlockCache* GDALArrayBandBlockCacheCreate(GDALRasterBand* poBand
 /************************************************************************/
 
 GDALArrayBandBlockCache::GDALArrayBandBlockCache(GDALRasterBand* poBandIn) :
-    GDALAbstractBandBlockCache(poBandIn),
-    bSubBlockingActive(false),
-    nSubBlocksPerRow(0),
-    nSubBlocksPerColumn(0)
+    GDALAbstractBandBlockCache(poBandIn)
 {
-    u.papoBlocks = nullptr;
 }
 
 /************************************************************************/
@@ -256,6 +257,8 @@ CPLErr GDALArrayBandBlockCache::FlushCache()
 
     CPLErr eGlobalErr = poBand->eFlushBlockErr;
 
+    StartDirtyBlockFlushingLog();
+
 /* -------------------------------------------------------------------- */
 /*      Flush all blocks in memory ... this case is without subblocking.*/
 /* -------------------------------------------------------------------- */
@@ -317,6 +320,8 @@ CPLErr GDALArrayBandBlockCache::FlushCache()
         }
     }
 
+    EndDirtyBlockFlushingLog();
+
     WaitKeepAliveCounter();
 
     return( eGlobalErr );
@@ -384,6 +389,7 @@ CPLErr GDALArrayBandBlockCache::FlushBlock( int nXBlockOff, int nYBlockOff,
     {
         const int nBlockIndex = nXBlockOff + nYBlockOff * poBand->nBlocksPerRow;
 
+        assert(u.papoBlocks);
         poBlock = u.papoBlocks[nBlockIndex];
         u.papoBlocks[nBlockIndex] = nullptr;
     }
@@ -422,8 +428,13 @@ CPLErr GDALArrayBandBlockCache::FlushBlock( int nXBlockOff, int nYBlockOff,
     poBlock->Detach();
 
     CPLErr eErr = CE_None;
+
     if( bWriteDirtyBlock && poBlock->GetDirty() )
+    {
+        UpdateDirtyBlockFlushingLog();
+
         eErr = poBlock->Write();
+    }
 
 /* -------------------------------------------------------------------- */
 /*      Deallocate the block;                                           */
