@@ -33,7 +33,9 @@
 #include "cpl_csv.h"
 #include "ogr_p.h"
 
-CPL_CVSID("$Id: ogr_srs_panorama.cpp 971ad299681ca1ea2e1b800e88209f426b77e9aa 2018-04-17 12:14:43 +0200 Even Rouault $")
+#include <cmath>
+
+CPL_CVSID("$Id: ogr_srs_panorama.cpp 8e5eeb35bf76390e3134a4ea7076dab7d478ea0e 2018-11-14 22:55:13 +0100 Even Rouault $")
 
 constexpr double TO_DEGREES = 57.2957795130823208766;
 constexpr double TO_RADIANS = 0.017453292519943295769;
@@ -459,9 +461,8 @@ OGRErr OGRSpatialReference::importFromPanorama( long iProjSys, long iDatum,
             else
             {
                 CPLError( CE_Warning, CPLE_AppDefined,
-                          "Failed to lookup ellipsoid code %ld, likely due to "
-                          "missing GDAL gcs.csv "
-                          "file.  Falling back to use Pulkovo 42.", iEllips );
+                          "Failed to lookup ellipsoid code %ld. "
+                          "Falling back to use Pulkovo 42.", iEllips );
                 SetWellKnownGeogCS( "EPSG:4284" );
             }
 
@@ -483,8 +484,6 @@ OGRErr OGRSpatialReference::importFromPanorama( long iProjSys, long iDatum,
     if( IsLocal() || IsProjected() )
         SetLinearUnits( SRS_UL_METER, 1.0 );
 
-    FixupOrdering();
-
     if( bProjAllocated && padfPrjParams )
         CPLFree( padfPrjParams );
 
@@ -805,8 +804,8 @@ OGRErr OGRSpatialReference::exportToPanorama( long *piProjSys, long *piDatum,
 
                 if( OSRGetEllipsoidInfo( aoEllips[i], nullptr,
                                          &dfSM, &dfIF ) == OGRERR_NONE
-                    && CPLIsEqual(dfSemiMajor, dfSM)
-                    && CPLIsEqual(dfInvFlattening, dfIF) )
+                    && std::abs(dfSemiMajor - dfSM) < 1e-10 * dfSemiMajor
+                    && std::abs(dfInvFlattening - dfIF) < 1e-10 * dfInvFlattening )
                 {
                     *piEllips = i;
                     break;
@@ -818,7 +817,7 @@ OGRErr OGRSpatialReference::exportToPanorama( long *piProjSys, long *piDatum,
         {
 #ifdef DEBUG
             CPLDebug( "OSR_Panorama",
-                      "Ellipsoid \"%s\" unsupported by \"Panorama\" GIS.",
+                      R"(Ellipsoid "%s" unsupported by "Panorama" GIS.)",
                       pszDatum );
 #endif
             *piDatum = PAN_DATUM_NONE;
