@@ -1,5 +1,5 @@
 /******************************************************************************
- * $Id: ogrgeojsonreader.h e4041afa8c4a60ed29ae460fd764b20bf1a5b4a7 2018-09-17 00:07:16 +0200 Even Rouault $
+ * $Id: ogrgeojsonreader.h 8b5aa2265f771dc327f6344c13c14ed16895fc29 2019-05-02 11:16:35 +0200 Even Rouault $
  *
  * Project:  OpenGIS Simple Features Reference Implementation
  * Purpose:  Defines GeoJSON reader within OGR OGRGeoJSON Driver.
@@ -37,6 +37,7 @@
 
 #include "ogrgeojsonutils.h"
 
+#include <utility>
 #include <set>
 
 /************************************************************************/
@@ -85,17 +86,13 @@ struct GeoJSONObject
 };
 
 /************************************************************************/
-/*                           OGRGeoJSONReader                           */
+/*                        OGRGeoJSONBaseReader                          */
 /************************************************************************/
 
-class OGRGeoJSONDataSource;
-class OGRGeoJSONReaderStreamingParser;
-
-class OGRGeoJSONReader
+class OGRGeoJSONBaseReader
 {
   public:
-    OGRGeoJSONReader();
-    ~OGRGeoJSONReader();
+      OGRGeoJSONBaseReader();
 
     void SetPreserveGeometryType( bool bPreserve );
     void SetSkipAttributes( bool bSkip );
@@ -103,6 +100,55 @@ class OGRGeoJSONReader
     void SetStoreNativeData( bool bStoreNativeData );
     void SetArrayAsString( bool bArrayAsString );
 
+    bool GenerateFeatureDefn( OGRLayer* poLayer, json_object* poObj );
+    void FinalizeLayerDefn( OGRLayer* poLayer, CPLString& osFIDColumn );
+
+    OGRGeometry* ReadGeometry( json_object* poObj, OGRSpatialReference* poLayerSRS );
+    OGRFeature* ReadFeature( OGRLayer* poLayer, json_object* poObj,
+                             const char* pszSerializedObj );
+  protected:
+    bool bGeometryPreserve_ = true;
+    bool bAttributesSkip_ = false;
+    bool bFlattenNestedAttributes_ = false;
+    char chNestedAttributeSeparator_ = 0;
+    bool bStoreNativeData_ = false;
+    bool bArrayAsString_ = false;
+
+  private:
+
+    std::set<int> aoSetUndeterminedTypeFields_;
+
+    // bFlatten... is a tri-state boolean with -1 being unset.
+    int bFlattenGeocouchSpatiallistFormat = -1;
+
+    bool bFoundGeocouchId = false;
+    bool bFoundRev = false;
+    bool bFoundTypeFeature = false;
+    bool bIsGeocouchSpatiallistFormat = false;
+    bool bFeatureLevelIdAsAttribute_ = false;
+    bool bFeatureLevelIdAsFID_ = false;
+    bool m_bNeedFID64 = false;
+
+    bool m_bDetectLayerGeomType = true;
+    bool m_bFirstGeometry = true;
+    OGRwkbGeometryType m_eLayerGeomType = wkbUnknown;
+
+    CPL_DISALLOW_COPY_ASSIGN(OGRGeoJSONBaseReader)
+};
+
+/************************************************************************/
+/*                           OGRGeoJSONReader                           */
+/************************************************************************/
+
+class OGRGeoJSONDataSource;
+class OGRGeoJSONReaderStreamingParser;
+
+class OGRGeoJSONReader: public OGRGeoJSONBaseReader
+{
+  public:
+    OGRGeoJSONReader();
+    ~OGRGeoJSONReader();
+
     OGRErr Parse( const char* pszText );
     void ReadLayers( OGRGeoJSONDataSource* poDS );
     void ReadLayer( OGRGeoJSONDataSource* poDS,
@@ -115,6 +161,7 @@ class OGRGeoJSONReader
 
     void ResetReading();
     OGRFeature* GetNextFeature(OGRGeoJSONLayer* poLayer);
+    OGRFeature* GetFeature(OGRGeoJSONLayer* poLayer, GIntBig nFID);
     bool IngestAll(OGRGeoJSONLayer* poLayer);
 
     VSILFILE* GetFP() { return fp_; }
@@ -132,48 +179,26 @@ class OGRGeoJSONReader
     bool bCanEasilyAppend_;
     bool bFCHasBBOX_;
 
-    bool bGeometryPreserve_;
-    bool bAttributesSkip_;
-    bool bFlattenNestedAttributes_;
-    char chNestedAttributeSeparator_;
-    bool bStoreNativeData_;
-    bool bArrayAsString_;
-    std::set<int> aoSetUndeterminedTypeFields_;
-
     size_t nBufferSize_;
     GByte* pabyBuffer_;
 
     GIntBig nTotalFeatureCount_;
     GUIntBig nTotalOGRFeatureMemEstimate_;
 
-    // bFlatten... is a tri-state boolean with -1 being unset.
-    int bFlattenGeocouchSpatiallistFormat;
-
-    bool bFoundGeocouchId;
-    bool bFoundRev;
-    bool bFoundTypeFeature;
-    bool bIsGeocouchSpatiallistFormat;
-    bool bFeatureLevelIdAsFID_;
-    bool bFeatureLevelIdAsAttribute_;
-
+    std::map<GIntBig, std::pair<vsi_l_offset, vsi_l_offset>> oMapFIDToOffsetSize_;
     //
     // Copy operations not supported.
     //
-    OGRGeoJSONReader( OGRGeoJSONReader const& );
-    OGRGeoJSONReader& operator=( OGRGeoJSONReader const& );
+    CPL_DISALLOW_COPY_ASSIGN(OGRGeoJSONReader)
 
     //
     // Translation utilities.
     //
     bool GenerateLayerDefn( OGRGeoJSONLayer* poLayer, json_object* poGJObject );
-    bool GenerateFeatureDefn( OGRGeoJSONLayer* poLayer, json_object* poObj );
-    void FinalizeLayerDefn( OGRGeoJSONLayer* poLayer );
+
     static bool AddFeature( OGRGeoJSONLayer* poLayer, OGRGeometry* poGeometry );
     static bool AddFeature( OGRGeoJSONLayer* poLayer, OGRFeature* poFeature );
 
-    OGRGeometry* ReadGeometry( json_object* poObj, OGRSpatialReference* poLayerSRS );
-    OGRFeature* ReadFeature( OGRGeoJSONLayer* poLayer, json_object* poObj,
-                             const char* pszSerializedObj );
     void ReadFeatureCollection( OGRGeoJSONLayer* poLayer, json_object* poObj );
     size_t SkipPrologEpilogAndUpdateJSonPLikeWrapper( size_t nRead );
 };
@@ -213,6 +238,11 @@ json_object CPL_DLL*  CPL_json_object_object_get( struct json_object* obj,
 bool CPL_DLL OGRJSonParse( const char* pszText, json_object** ppoObj,
                            bool bVerboseError = true );
 
+bool OGRGeoJSONUpdateLayerGeomType( OGRLayer* poLayer,
+                                    bool& bFirstGeom,
+                                    OGRwkbGeometryType eGeomType,
+                                    OGRwkbGeometryType& eLayerGeomType );
+
 /************************************************************************/
 /*                 GeoJSON Geometry Translators                         */
 /************************************************************************/
