@@ -29,14 +29,14 @@
 
 #include "ogr_mssqlspatial.h"
 
-CPL_CVSID("$Id: ogrmssqlspatialdatasource.cpp eb9f38ed0b59555a745cf11fe42ae16f64b58b3c 2018-05-13 22:25:08 +0200 Tamas Szekeres $")
+CPL_CVSID("$Id: ogrmssqlspatialdatasource.cpp 74bf9367d33c608de344cc32a57a9ca924e36191 2019-03-31 20:45:06 +0200 Tamas Szekeres $")
 
 /************************************************************************/
 /*                          OGRMSSQLSpatialDataSource()                 */
 /************************************************************************/
 
 OGRMSSQLSpatialDataSource::OGRMSSQLSpatialDataSource() :
-    bDSUpdate(FALSE)
+    bDSUpdate(false)
 {
     pszName = nullptr;
     pszCatalog = nullptr;
@@ -52,7 +52,13 @@ OGRMSSQLSpatialDataSource::OGRMSSQLSpatialDataSource() :
     nGeometryFormat = MSSQLGEOMETRY_NATIVE;
     pszConnection = nullptr;
 
+    sMSSQLVersion.nMajor = 0;
+    sMSSQLVersion.nMinor = 0;
+    sMSSQLVersion.nBuild = 0;
+    sMSSQLVersion.nRevision = 0;
+
     bUseGeometryColumns = CPLTestBool(CPLGetConfigOption("MSSQLSPATIAL_USE_GEOMETRY_COLUMNS", "YES"));
+    bAlwaysOutputFid = CPLTestBool(CPLGetConfigOption("MSSQLSPATIAL_ALWAYS_OUTPUT_FID", "NO"));
     bListAllTables = CPLTestBool(CPLGetConfigOption("MSSQLSPATIAL_LIST_ALL_TABLES", "NO"));
 
     const char* nBCPSizeParam = CPLGetConfigOption("MSSQLSPATIAL_BCP_SIZE", nullptr);
@@ -65,6 +71,7 @@ OGRMSSQLSpatialDataSource::OGRMSSQLSpatialDataSource() :
 #else
     bUseCopy = FALSE;
 #endif
+    CPLDebug( "MSSQLSpatial", "Use COPY/BCP: %d", bUseCopy );
 }
 
 /************************************************************************/
@@ -91,6 +98,70 @@ OGRMSSQLSpatialDataSource::~OGRMSSQLSpatialDataSource()
     CPLFree( papoSRS );
     CPLFree( pszConnection );
 }
+/************************************************************************/
+/*                      OGRMSSQLDecodeVersionString()                   */
+/************************************************************************/
+
+void OGRMSSQLSpatialDataSource::OGRMSSQLDecodeVersionString(MSSQLVer* psVersion, const char* pszVer)
+{
+    while (*pszVer == ' ') pszVer++;
+
+    const char* ptr = pszVer;
+    // get Version string
+    while (*ptr && *ptr != ' ') ptr++;
+    GUInt32 iLen = static_cast<int>(ptr - pszVer);
+    char szVer[20] = {};
+    if (iLen > sizeof(szVer) - 1) iLen = sizeof(szVer) - 1;
+    strncpy(szVer, pszVer, iLen);
+    szVer[iLen] = '\0';
+
+    ptr = pszVer = szVer;
+
+    // get Major number
+    while (*ptr && *ptr != '.') ptr++;
+    iLen = static_cast<int>(ptr - pszVer);
+    char szNum[20] = {};
+    if (iLen > sizeof(szNum) - 1) iLen = sizeof(szNum) - 1;
+    strncpy(szNum, pszVer, iLen);
+    szNum[iLen] = '\0';
+    psVersion->nMajor = atoi(szNum);
+
+    if (*ptr == 0)
+        return;
+    pszVer = ++ptr;
+
+    // get Minor number
+    while (*ptr && *ptr != '.') ptr++;
+    iLen = static_cast<int>(ptr - pszVer);
+    if (iLen > sizeof(szNum) - 1) iLen = sizeof(szNum) - 1;
+    strncpy(szNum, pszVer, iLen);
+    szNum[iLen] = '\0';
+    psVersion->nMinor = atoi(szNum);
+
+    if (*ptr == 0)
+        return;
+    pszVer = ++ptr;
+
+    // get Build number
+    while (*ptr && *ptr != '.') ptr++;
+    iLen = static_cast<int>(ptr - pszVer);
+    if (iLen > sizeof(szNum) - 1) iLen = sizeof(szNum) - 1;
+    strncpy(szNum, pszVer, iLen);
+    szNum[iLen] = '\0';
+    psVersion->nBuild = atoi(szNum);
+
+    if (*ptr == 0)
+        return;
+    pszVer = ++ptr;
+
+    // get Revision number
+    while (*ptr && *ptr != '.') ptr++;
+    iLen = static_cast<int>(ptr - pszVer);
+    if (iLen > sizeof(szNum) - 1) iLen = sizeof(szNum) - 1;
+    strncpy(szNum, pszVer, iLen);
+    szNum[iLen] = '\0';
+    psVersion->nRevision = atoi(szNum);
+}
 
 /************************************************************************/
 /*                           TestCapability()                           */
@@ -107,6 +178,12 @@ int OGRMSSQLSpatialDataSource::TestCapability( const char * pszCap )
         return TRUE;
     if( EQUAL(pszCap,ODsCRandomLayerWrite) )
         return TRUE;
+    if (EQUAL(pszCap, OLCFastGetExtent))
+        return TRUE;
+    else if (EQUAL(pszCap, ODsCCurveGeometries))
+        return TRUE;
+    else if (EQUAL(pszCap, ODsCMeasuredGeometries))
+        return TRUE;
     else
         return FALSE;
 }
@@ -528,7 +605,7 @@ OGRLayer * OGRMSSQLSpatialDataSource::ICreateLayer( const char * pszLayerName,
 int OGRMSSQLSpatialDataSource::OpenTable( const char *pszSchemaName, const char *pszTableName,
                                           const char *pszGeomCol, int nCoordDimension,
                                           int nSRID, const char *pszSRText, OGRwkbGeometryType eType,
-                                          CPL_UNUSED int bUpdate )
+                                          bool bUpdate )
 {
 /* -------------------------------------------------------------------- */
 /*      Create the layer object.                                        */
@@ -540,6 +617,7 @@ int OGRMSSQLSpatialDataSource::OpenTable( const char *pszSchemaName, const char
         delete poLayer;
         return FALSE;
     }
+    poLayer->SetUpdate(bUpdate);
 
     if (bUseCopy)
         poLayer->SetUseCopy(nBCPSize);
@@ -594,7 +672,7 @@ int OGRMSSQLSpatialDataSource::ParseValue(char** pszValue, char* pszSource, cons
 /*                                Open()                                */
 /************************************************************************/
 
-int OGRMSSQLSpatialDataSource::Open( const char * pszNewName, int bUpdate,
+int OGRMSSQLSpatialDataSource::Open( const char * pszNewName, bool bUpdate,
                              int bTestOpen )
 
 {
@@ -752,6 +830,10 @@ int OGRMSSQLSpatialDataSource::Open( const char * pszNewName, int bUpdate,
         pszDriver = CPLStrdup("{SQL Server Native Client 11.0}");
 #elif SQLNCLI_VERSION == 10
         pszDriver = CPLStrdup("{SQL Server Native Client 10.0}");
+#elif MSODBCSQL_VERSION == 13
+        pszDriver = CPLStrdup("{ODBC Driver 13 for SQL Server}");
+#elif MSODBCSQL_VERSION == 17
+        pszDriver = CPLStrdup("{ODBC Driver 17 for SQL Server}");
 #else
         pszDriver = CPLStrdup("{SQL Server}");
 #endif
@@ -808,6 +890,29 @@ int OGRMSSQLSpatialDataSource::Open( const char * pszNewName, int bUpdate,
         return FALSE;
     }
 
+    /* -------------------------------------------------------------------- */
+    /*      Find out SQLServer version                                      */
+    /* -------------------------------------------------------------------- */
+    if (true) {
+        sMSSQLVersion.nMajor = -1;
+        sMSSQLVersion.nMinor = -1;
+        sMSSQLVersion.nBuild = -1;
+        sMSSQLVersion.nRevision = -1;
+
+        CPLODBCStatement oStmt(&oSession);
+
+        /* Use join to make sure the existence of the referred column/table */
+        oStmt.Append("SELECT SERVERPROPERTY('ProductVersion') AS ProductVersion;");
+
+        if (oStmt.ExecuteSQL())
+        {
+            while (oStmt.Fetch())
+            {
+                OGRMSSQLDecodeVersionString(&sMSSQLVersion, oStmt.GetColData(0));
+            }
+        }
+    }
+
     char** papszTypes = nullptr;
 
     /* read metadata for the specified tables */
@@ -939,7 +1044,7 @@ int OGRMSSQLSpatialDataSource::Open( const char * pszNewName, int bUpdate,
         if (papszSRIds != nullptr)
             nSRId = atoi(papszSRIds[iTable]);
         else
-            nSRId = -1;
+            nSRId = 0;
 
         if (papszCoordDimensions != nullptr)
             nCoordDimension = atoi(papszCoordDimensions[iTable]);
@@ -951,6 +1056,7 @@ int OGRMSSQLSpatialDataSource::Open( const char * pszNewName, int bUpdate,
         else
             eType = wkbUnknown;
 
+        CPLAssert(papszGeomColumnNames && papszGeomColumnNames[iTable]);
         if( strlen(papszGeomColumnNames[iTable]) > 0 )
             OpenTable( papszSchemaNames[iTable], papszTableNames[iTable], papszGeomColumnNames[iTable],
                     nCoordDimension, nSRId, papszSRTexts? papszSRTexts[iTable] : nullptr, eType, bUpdate );
@@ -1212,12 +1318,25 @@ OGRSpatialReference *OGRMSSQLSpatialDataSource::FetchSRS( int nId )
             if ( oStmt.GetColData( 0 ) )
             {
                 poSRS = new OGRSpatialReference();
+                poSRS->SetAxisMappingStrategy(OAMS_TRADITIONAL_GIS_ORDER);
                 const char* pszWKT = oStmt.GetColData( 0 );
                 if( poSRS->importFromWkt( pszWKT ) != OGRERR_NONE )
                 {
                     delete poSRS;
                     poSRS = nullptr;
                 }
+                else
+                {
+                    const char* pszAuthorityName = poSRS->GetAuthorityName(nullptr);
+                    const char* pszAuthorityCode = poSRS->GetAuthorityCode(nullptr);
+                    if( pszAuthorityName && pszAuthorityCode &&
+                        EQUAL(pszAuthorityName, "EPSG") )
+                    {
+                        const int nCode = atoi(pszAuthorityCode);
+                        poSRS->Clear();
+                        poSRS->importFromEPSG(nCode);
+                    }
+                }
             }
         }
     }
@@ -1228,6 +1347,7 @@ OGRSpatialReference *OGRMSSQLSpatialDataSource::FetchSRS( int nId )
     if (!poSRS)
     {
         poSRS = new OGRSpatialReference();
+        poSRS->SetAxisMappingStrategy(OAMS_TRADITIONAL_GIS_ORDER);
         if( poSRS->importFromEPSG( nId ) != OGRERR_NONE )
         {
             delete poSRS;
